@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id: ogrmerge.py acb00496e69f7896f6b4453e9f49d2f2ce3e7b64 2018-04-27 00:56:06 +0200 Even Rouault $
+# $Id: ogrmerge.py 545d0423e15cc28a5aca3a9852fab80b80601162 2018-12-12 11:49:36 +1300 Craig de Stigter $
 #
 # Project:  GDAL/OGR samples
 # Purpose:  Merge the content of several vector datasets into a single one.
@@ -91,7 +91,7 @@ def GetOutputDriversFor(filename):
         if (drv.GetMetadataItem(gdal.DCAP_CREATE) is not None or
             drv.GetMetadataItem(gdal.DCAP_CREATECOPY) is not None) and \
            drv.GetMetadataItem(gdal.DCAP_VECTOR) is not None:
-            if len(ext) > 0 and DoesDriverHandleExtension(drv, ext):
+            if ext and DoesDriverHandleExtension(drv, ext):
                 drv_list.append(drv.ShortName)
             else:
                 prefix = drv.GetMetadataItem(gdal.DMD_CONNECTION_PREFIX)
@@ -103,9 +103,9 @@ def GetOutputDriversFor(filename):
 
 def GetOutputDriverFor(filename):
     drv_list = GetOutputDriversFor(filename)
-    if len(drv_list) == 0:
-        ext = GetExtension(filename)
-        if len(ext) == 0:
+    ext = GetExtension(filename)
+    if not drv_list:
+        if not ext:
             return 'ESRI Shapefile'
         else:
             raise Exception("Cannot guess driver for %s" % filename)
@@ -149,7 +149,7 @@ def _Esc(x):
     return gdal.EscapeString(x, gdal.CPLES_XML)
 
 
-class XMLWriter:
+class XMLWriter(object):
 
     def __init__(self, f):
         self.f = f
@@ -157,14 +157,16 @@ class XMLWriter:
         self.elements = []
 
     def _indent(self):
-        return ''.join(['  ' for i in range(self.inc)])
+        return '  ' * self.inc
 
     def open_element(self, name, attrs=None):
         xml_attrs = ''
         if attrs is not None:
             for key in attrs:
-                xml_attrs = xml_attrs + ' %s=\"%s\"' % (key, _Esc(attrs[key]))
-        _VSIFPrintfL(self.f, '%s<%s%s>\n' % (self._indent(), name, xml_attrs))
+                xml_attrs = xml_attrs + ' %s=\"%s\"' % (key, _Esc(attrs[key].encode('utf-8')))
+        x = '%s<%s%s>\n' % (self._indent(), name, xml_attrs)
+        x = x.encode('utf-8')
+        _VSIFPrintfL(self.f, x)
         self.inc = self.inc + 1
         self.elements.append(name)
 
@@ -172,10 +174,11 @@ class XMLWriter:
         xml_attrs = ''
         if attrs is not None:
             for key in attrs:
-                xml_attrs = xml_attrs + ' %s=\"%s\"' % (key, _Esc(attrs[key]))
-        _VSIFPrintfL(self.f, '%s<%s%s>%s</%s>\n' %
-                     (self._indent(), name, xml_attrs,
-                      _Esc(value), name))
+                xml_attrs = xml_attrs + ' %s=\"%s\"' % (key, _Esc(attrs[key].encode('utf-8')))
+        x = '%s<%s%s>%s</%s>\n' % (self._indent(), name, xml_attrs,
+                      _Esc(value.encode('utf-8')), name)
+        x = x.encode('utf-8')
+        _VSIFPrintfL(self.f, x)
 
     def close_element(self, closing_name=None):
         self.inc = self.inc - 1
@@ -192,7 +195,7 @@ class XMLWriter:
 
 def process(argv, progress=None, progress_arg=None):
 
-    if len(argv) == 0:
+    if not argv:
         return Usage()
 
     dst_filename = None
@@ -285,7 +288,10 @@ def process(argv, progress=None, progress_arg=None):
             return Usage()
         else:
             if '*' in arg:
-                src_datasets += glob.glob(arg)
+                if sys.version_info < (3,0,0):
+                    src_datasets += [fn.decode(sys.getfilesystemencoding()) for fn in glob.glob(arg)]
+                else:
+                    src_datasets += glob.glob(arg)
             else:
                 src_datasets.append(arg)
         i = i + 1
@@ -298,7 +304,7 @@ def process(argv, progress=None, progress_arg=None):
         if output_format is not None:
             print('ERROR: -f incompatible with -update')
             return 1
-        if len(dsco) != 0:
+        if dsco:
             print('ERROR: -dsco incompatible with -update')
             return 1
         output_format = ''
@@ -317,7 +323,7 @@ def process(argv, progress=None, progress_arg=None):
               'shapefile output')
         return 1
 
-    if len(src_datasets) == 0:
+    if not src_datasets:
         print('ERROR: No source datasets')
         return 1
 
@@ -386,7 +392,7 @@ def process(argv, progress=None, progress_arg=None):
                 gdal.Unlink(vrt_filename)
                 return 1
             for src_lyr_idx, src_lyr in enumerate(src_ds):
-                if len(src_geom_types) != 0:
+                if src_geom_types:
                     gt = ogr.GT_Flatten(src_lyr.GetGeomType())
                     if gt not in src_geom_types:
                         continue
@@ -406,21 +412,27 @@ def process(argv, progress=None, progress_arg=None):
 
                 layer_name = src_layer_field_content
 
+                src_lyr_name = src_lyr.GetName()
+                try:
+                    src_lyr_name = src_lyr_name.decode('utf-8')
+                except AttributeError:
+                    pass
+
                 basename = None
                 if os.path.exists(src_dsname):
                     basename = os.path.basename(src_dsname)
-                    if basename.find('.') >= 0:
+                    if '.' in basename:
                         basename = '.'.join(basename.split(".")[0:-1])
 
-                if basename == src_lyr.GetName():
+                if basename == src_lyr_name:
                     layer_name = layer_name.replace('{AUTO_NAME}', basename)
                 elif basename is None:
                     layer_name = layer_name.replace(
                         '{AUTO_NAME}',
-                        'Dataset%d_%s' % (src_ds_idx, src_lyr.GetName()))
+                        'Dataset%d_%s' % (src_ds_idx, src_lyr_name))
                 else:
                     layer_name = layer_name.replace(
-                        '{AUTO_NAME}', basename + '_' + src_lyr.GetName())
+                        '{AUTO_NAME}', basename + '_' + src_lyr_name)
 
                 if basename is not None:
                     layer_name = layer_name.replace('{DS_BASENAME}', basename)
@@ -432,7 +444,7 @@ def process(argv, progress=None, progress_arg=None):
                 layer_name = layer_name.replace('{DS_INDEX}', '%d' %
                                                 src_ds_idx)
                 layer_name = layer_name.replace('{LAYER_NAME}',
-                                                src_lyr.GetName())
+                                                src_lyr_name)
                 layer_name = layer_name.replace('{LAYER_INDEX}', '%d' %
                                                 src_lyr_idx)
 
@@ -445,8 +457,8 @@ def process(argv, progress=None, progress_arg=None):
                 if EQUAL(output_format, 'VRT') and \
                    os.path.exists(src_dsname) and \
                    not os.path.isabs(src_dsname) and \
-                   vrt_filename.find('/') < 0 and \
-                   vrt_filename.find('\\') < 0:
+                   '/' not in vrt_filename and \
+                   '\\' not in vrt_filename:
                     attrs = {'relativeToVRT': '1'}
                 writer.write_element_value('SrcDataSource', src_dsname,
                                            attrs=attrs)
@@ -480,33 +492,39 @@ def process(argv, progress=None, progress_arg=None):
                 gdal.Unlink(vrt_filename)
                 return 1
             for src_lyr_idx, src_lyr in enumerate(src_ds):
-                if len(src_geom_types) != 0:
+                if src_geom_types:
                     gt = ogr.GT_Flatten(src_lyr.GetGeomType())
                     if gt not in src_geom_types:
                         continue
 
+                src_lyr_name = src_lyr.GetName()
+                try:
+                    src_lyr_name = src_lyr_name.decode('utf-8')
+                except AttributeError:
+                    pass
+
                 layer_name = layer_name_template
                 basename = None
                 if os.path.exists(src_dsname):
                     basename = os.path.basename(src_dsname)
-                    if basename.find('.') >= 0:
+                    if '.' in basename:
                         basename = '.'.join(basename.split(".")[0:-1])
 
-                if basename == src_lyr.GetName():
+                if basename == src_lyr_name:
                     layer_name = layer_name.replace('{AUTO_NAME}', basename)
                 elif basename is None:
                     layer_name = layer_name.replace(
                         '{AUTO_NAME}',
-                        'Dataset%d_%s' % (src_ds_idx, src_lyr.GetName()))
+                        'Dataset%d_%s' % (src_ds_idx, src_lyr_name))
                 else:
                     layer_name = layer_name.replace(
-                        '{AUTO_NAME}', basename + '_' + src_lyr.GetName())
+                        '{AUTO_NAME}', basename + '_' + src_lyr_name)
 
                 if basename is not None:
                     layer_name = layer_name.replace('{DS_BASENAME}', basename)
-                elif layer_name.find('{DS_BASENAME}') >= 0:
+                elif '{DS_BASENAME}' in layer_name:
                     if skip_failures:
-                        if layer_name.find('{DS_INDEX}') < 0:
+                        if '{DS_INDEX}' not in layer_name:
                             layer_name = layer_name.replace(
                                 '{DS_BASENAME}', 'Dataset%d' % src_ds_idx)
                     else:
@@ -523,7 +541,7 @@ def process(argv, progress=None, progress_arg=None):
                 layer_name = layer_name.replace('{DS_INDEX}', '%d' %
                                                 src_ds_idx)
                 layer_name = layer_name.replace('{LAYER_NAME}',
-                                                src_lyr.GetName())
+                                                src_lyr_name)
                 layer_name = layer_name.replace('{LAYER_INDEX}', '%d' %
                                                 src_lyr_idx)
 
@@ -536,12 +554,12 @@ def process(argv, progress=None, progress_arg=None):
                 if EQUAL(output_format, 'VRT') and \
                    os.path.exists(src_dsname) and \
                    not os.path.isabs(src_dsname) and \
-                   vrt_filename.find('/') < 0 and \
-                   vrt_filename.find('\\') < 0:
+                   '/' not in vrt_filename and \
+                   '\\' not in vrt_filename:
                     attrs = {'relativeToVRT': '1'}
                 writer.write_element_value('SrcDataSource', src_dsname,
                                            attrs=attrs)
-                writer.write_element_value('SrcLayer', src_lyr.GetName())
+                writer.write_element_value('SrcLayer', src_lyr_name)
 
                 if a_srs is not None:
                     writer.write_element_value('LayerSRS', a_srs)
@@ -586,7 +604,10 @@ def process(argv, progress=None, progress_arg=None):
 
 
 def main():
-    argv = ogr.GeneralCmdLineProcessor(sys.argv)
+    argv = sys.argv
+    if sys.version_info < (3,0,0):
+        argv = [fn.decode(sys.getfilesystemencoding()) for fn in argv]
+    argv = ogr.GeneralCmdLineProcessor(argv)
     if argv is None:
         return 1
     return process(argv[1:])
